classdef pVUS < VUS
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        p1
        p2
        q1
        q2
        
        c1_p1
        c1_p2
        c2_q1
        c2_q2  
        
        isvalid
        pvus
        pvus_scaled
        isvalidH0
        pvusH0
% Grid of TC1s between p1 and p2        
        tc1gr_p
% Grid of TC2s for TC1s between p1 and p2 and TC3s between q1 and q2       
        tc2gr_pq
% Grid of TC3s between q1 and q2   
        tc3gr_q
        c1gr_p
        c2gr_q
    end
    
    methods
        function obj = pVUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,p1,p2,q1,q2,gr_sz)
            %UNTITLED Construct an instance of this class
            %   Detailed explanation goes here
            if nargin < 14; gr_sz = 0.0025; end
            obj@VUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,gr_sz);

            obj.p1 = p1;
            obj.p2 = p2;
            obj.q1 = q1;
            obj.q2 = q2;
            
            [obj.pvusH0, obj.isvalidH0] = pVUS.calcPVUSH0(p1, p2, q1, q2);
            
            % Set cutoff point values at the boundaries of the pVUS
            obj.c1_p1 = obj.F1inv(obj.p1);   obj.c1_p2 = obj.F1inv(obj.p2);
            obj.c2_q1 = obj.F3inv(1-obj.q1); obj.c2_q2 = obj.F3inv(1-obj.q2);
            
            [obj.pvus, obj.isvalid] = pVUS.calcPVUS(F1, F2, F3, F1inv, F3inv, f2, p1, p2, q1, q2);
            obj.pvus_scaled = obj.pvus./(p2-p1)./(q2-q1);
            if gr_sz < 1
                tc2 = @(c1, c2) (obj.F2(c2) - obj.F2(c1)).*(c2 > c1);
                % Set partial grid for plotting partial ROC Surface
                obj.tc1gr_p = [obj.p1:gr_sz:obj.p2 obj.p2];
                obj.tc3gr_q = [obj.q1:gr_sz:obj.q2 obj.q2];
                obj.c1gr_p = obj.F1inv(obj.tc1gr_p);
                obj.c2gr_q = obj.F3inv(1-obj.tc3gr_q);
                [c1_m, c2_m] = meshgrid(obj.c1gr_p, obj.c2gr_q);
                sz = size(c1_m);
                Z = tc2(reshape(c1_m, 1, []),reshape(c2_m, 1, []));
                obj.tc2gr_pq = reshape(Z, sz(1), sz(2));
                obj.tc2gr_pq(obj.tc2gr_pq == 0) = -0.1;
            end
        end        
%-------------------------------------------------------------------------%        
% Plot boundaries of pVUS on the x-y plane        
        function plotBounds(obj, varargin)
            plot([obj.p1 obj.p1], [0 1], varargin{:})
            plot([obj.p2 obj.p2], [0 1], varargin{:})
            plot([0 1], [obj.q1 obj.q1], varargin{:})
            plot([0 1], [obj.q2 obj.q2], varargin{:})
        end
%-------------------------------------------------------------------------% 
% Plot outline of floor
        function plotBoundFloor(obj, varargin)
            surf([0 1], [0 1], [0 0; 0 0], varargin{:})    
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function  pl = plotSurf(obj, varargin)
            pl = surf(obj.tc1gr_p, obj.tc3gr_q, obj.tc2gr_pq, varargin{:});
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function  pl = plotSurfH0(obj, varargin)
            X = [obj.p1; obj.p1; obj.p2; obj.p2];
            Y = [obj.q1; obj.q2; obj.q2; obj.q1];
            Z = 1-X-Y;
            pl = patch(X,Y,Z,'k', varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the Outline of the pVUS
        function plotOutline(obj, varargin)
            tc2 = @(c1, c2) (obj.F2(c2) - obj.F2(c1)).*(c2 > c1);
            % Line on ROC Surface from (p1,q1) to (p2,q1)
                X1 = obj.tc1gr_p;
                Y1 = X1*0 + obj.q1;
                C1 = obj.c1gr_p;
                C2 = X1*0 + obj.c2_q1;
                Z1 = tc2(C1, C2);
            % Line on ROC Surface from (p1,q2) to (p2,q2) 
                X2 = obj.tc1gr_p;
                Y2 = X2.*0 + obj.q2;
                C2 = X1*0 + obj.c2_q2;
                Z2 = tc2(C1, C2);
            % Line on ROC Surface from (p1,q1) to (p1,q2)
                Y3 = obj.tc3gr_q;
                X3 = Y3.*0 + obj.p1;
                C2 = obj.c2gr_q;
                C1 = Y3.*0 + obj.c1_p1;
                Z3 = tc2(C1, C2);
            % Line on ROC Surface from (p2,q1) to (p2,q2)
                Y4 = obj.tc3gr_q;
                X4 = Y4.*0 + obj.p2;
                C1 = Y4.*0 + obj.c1_p2;
                Z4 = tc2(C1, C2);
            hold on;
            plot3(X1, Y1, Z1, varargin{:});
            plot3(X2, Y2, Z2, varargin{:});
            plot3(X3, Y3, Z3, varargin{:});
            plot3(X4, Y4, Z4, varargin{:});
            plot3([obj.p1, obj.p2], [obj.q1, obj.q1], [0 0], varargin{:});
            plot3([obj.p1, obj.p1], [obj.q1, obj.q2], [0 0], varargin{:});
            plot3([obj.p1, obj.p2], [obj.q2, obj.q2], [0 0], varargin{:});
            plot3([obj.p2, obj.p2], [obj.q1, obj.q2], [0 0], varargin{:});
            plot3([obj.p1, obj.p1], [obj.q1, obj.q1], [0 tc2(obj.c1_p1, obj.c2_q1)], varargin{:});
            plot3([obj.p1, obj.p1], [obj.q2, obj.q2], [0 tc2(obj.c1_p1, obj.c2_q2)], varargin{:});
            plot3([obj.p2, obj.p2], [obj.q1, obj.q1], [0 tc2(obj.c1_p2, obj.c2_q1)], varargin{:});
            plot3([obj.p2, obj.p2], [obj.q2, obj.q2], [0 tc2(obj.c1_p2, obj.c2_q2)], varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function pl = plotSides(obj, varargin)
            tc2 = @(c1, c2) (obj.F2(obj.F3inv(1-c2)) - obj.F2(obj.F1inv(c1)));
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            X1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Y1 = [obj.q1; obj.q1;  p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            Z1 = [0; 0; tc2(p_gr',obj.q1); 0; 0];
            Y2 = [obj.q2; obj.q2;  p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            Z2 = [0; 0; tc2(p_gr',obj.q2); 0; 0];
            Y3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            X3 = [obj.p1; obj.p1;  q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            Z3 = [0; 0; tc2(obj.p1, q_gr'); 0; 0];
            X4 = [obj.p2; obj.p2;  q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            Z4 = [0; 0; tc2(obj.p2, q_gr'); 0; 0];
            C = Z4.*0 + 1;
            pl = patch([X1,X1,X3,X4], [Y1,Y2,Y3,Y3], [Z1,Z2,Z3,Z4], [C.*0, C.*0.6, C.*0.6, C.*0], varargin{:});
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p, r] = plotSurfWithROC(obj, varargin)
            tc2 = @(x, y) (obj.F2(obj.F3inv(1-y)) - obj.F2(obj.F1inv(x)));
            p = obj.plotSurf(varargin{:});
            x = unique([0:obj.gr_sz:obj.p1, obj.p1:obj.gr_sz:obj.p2, obj.p2:obj.gr_sz:1, 1]);
            y = unique([0:obj.gr_sz:obj.q1, obj.q1:obj.gr_sz:obj.q2, obj.q2:obj.gr_sz:1, 1]);
            [x_m, y_m] = meshgrid(x,y);
            z = tc2(x_m,y_m);
            z(y > obj.q1 & y < obj.q2, x > obj.p1 & x < obj.p2) = NaN;
            r = surf(x, y, z,'EdgeColor', 'none', 'FaceLighting', 'none');
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p r] = plotSurfWithROCH0(obj, varargin)
            p = obj.plotSurfH0(varargin{:});
            X = [1; 0; 0; obj.p1; obj.p1; obj.p2; obj.p2; obj.p1; 0];
            Y = [0; 1; 0; obj.q1; obj.q2; obj.q2; obj.q1; obj.q1; 0];
            Z = 1-X-Y;
            C = 1-X-Y;
            r = patch(X,Y,Z,C, 'EdgeColor','none','FaceLighting', 'none','FaceAlpha', 0.8);
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function plotSidesToROCH0(obj, varargin)
            tc2 = @(x, y) 1 - y - x;
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            X1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Y1 = [obj.q1; obj.q1;  p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            Z1 = [0; 0; tc2(p_gr',obj.q1); 0; 0];
            Y2 = [obj.q2; obj.q2;  p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            Z2 = [0; 0; tc2(p_gr',obj.q2); 0; 0];
            Y3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            X3 = [obj.p1; obj.p1;  q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            Z3 = [0; 0; tc2(obj.p1, q_gr'); 0; 0];
            X4 = [obj.p2; obj.p2;  q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            Z4 = [0; 0; tc2(obj.p2, q_gr'); 0; 0];
            patch([X1,X1,X3,X4], [Y1,Y2,Y3,Y3], [Z1,Z2,Z3,Z4], 'k', varargin{:})
            axis([0 1 0 1 0 1])
        end
%-------------------------------------------------------------------------%
% Plot the pVUS region on the x-y plane
        function plotFloor(obj, varargin)
            X = [obj.p1; obj.p2; obj.p2; obj.p1];
            Y = [obj.q1; obj.q1; obj.q2; obj.q2]; 
            Z = [0; 0; 0; 0];
            patch(X,Y,Z, varargin{:})
        end
        
        function plotROCandpVUS(obj)
            obj.plotROC('EdgeColor', 'none', 'FaceAlpha', 1)
            hold on
            obj.plotROCOutline('LineWidth', 1, 'Color', [1 1 1].*0.3)
            obj.plotSides('EdgeColor','none','FaceAlpha',0.6,'FaceColor','#9819E6')
            obj.plotFloor('EdgeColor','none','FaceColor','#420B63')
            obj.plotSurf(true, 'EdgeColor', 'none','FaceColor','#2A0740')
            obj.plotOutline('LineWidth', 1, 'Color', '#2D0845')
            obj.plotBounds('--k', 'Color', 'black') 
            light('Position',[0 0 2]);
            light('Position',[0 0 0.5],'Style','local');
            light('Position',[0 1 0.5],'Style','local');
        end
        function t = plotROCandPDFs(obj, ttl)
            t = tiledlayout(1,2);
            t.TileSpacing = 'compact';
            t.Padding = 'compact';
            nexttile
            obj.plotPDFs('Linewidth', 2)
            title('PDFs', 'FontWeight', 'normal');
            nexttile
            obj.plotROCandpVUS()
            title('ROC Surface', 'FontWeight', 'normal');
            title(t,ttl, 'FontWeight', 'bold')
        end
        function [pc1, pc2] = plotCutoffBounds(obj, c1Color, c2Color, varargin)
            Xrng = [999,-999];
            Finv{1} = obj.F1inv; Finv{2} = obj.F2inv; Finv{3} = obj.F3inv;
            for i = 1:3
                Xrng = [min(Finv{i}(0.001), Xrng(1)),max(Finv{i}(0.999), Xrng(2))];
            end
            x = linspace(Xrng(1),Xrng(2),1./obj.gr_sz);
            
            if obj.p1 == 0
                c1Lower = min(x);
            else
                c1Lower = obj.F1inv(obj.p1);
            end
            c1Upper = obj.F1inv(obj.p2);
            c2Lower = obj.F3inv(1-obj.q2);
            if obj.q1 == 0
                c2Upper = max(x);
            else
                c2Upper = obj.F3inv(1-obj.q1);
            end
            
            hold on;
            pc1 = patch([c1Lower, c1Lower, c1Upper, c1Upper], [-1 2 2 -1], 'g',...
                'FaceColor', c1Color, varargin{:});
            pc2 = patch([c2Upper, c2Upper, c2Lower, c2Lower],[-1 2 2 -1],'r',...
                'FaceColor', c2Color, varargin{:});
            ymax = max([obj.f1(x), obj.f2(x), obj.f3(x)]).*1.05;
            ylim([0, ymax]);
            hold off;
        end
%-------------------------------------------------------------------------%        
% Plot the Outline of the ROC surface on each plane        
        function pl = plotFeasReg(obj, varargin)
            x = obj.tc1gr;
            pl = area(x, 1-obj.F3(obj.c1gr), varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot the Outline of the ROC surface on each plane        
        function pl = plotFeasRegH0(obj, varargin)
            x = obj.tc1gr;
            pl = area(x, 1-x, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot the Outline of the ROC surface on each plane        
        function pl = plotFeasRegInf(obj, varargin)
            x = obj.tc1gr;
            y = 1-obj.F3(obj.c1gr);
            x(x > 1-y) = 0;
            pl = area(x,y,varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity regions on the x-y plane        
        function  plotFeasRegFlat(obj, varargin)
            x = obj.tc1gr;
            area(x, 1-obj.F3(obj.c1gr), varargin{:});
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_2$','Interpreter', 'Latex'); 
        end
        function plotFeasRegOutline(obj, varargin)
            x = obj.tc1gr;
            plot3(x, 1-obj.F3(obj.c1gr), x.*0, varargin{:});
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
    end
    methods(Static)
%-------------------------------------------------------------------------%
% Plot one side given values of x, y, and z (helper function for plotSides) 
        function s= plotSide(X, Y, Z, varargin)
            if length(unique(X)) == 1
                [Ym, Xm] = meshgrid(Y,X);
            else 
                [Xm, Ym] = meshgrid(X,Y);
            end

            steps = max(size(Z));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Zm = (start' * ones(1, steps) + Z'*t)';

            s = surf(Xm, Ym, Zm, varargin{:});
        end
        function s= plotSideToH0(X, Y, Z, Z0, varargin)
            if length(unique(X)) == 1
                [Ym, Xm] = meshgrid(Y,X);
            else 
                [Xm, Ym] = meshgrid(X,Y);
            end

            steps = max(size(Z));
            %start = zeros(1,steps);
            start = Z0;

            t = linspace(0,1,steps);
            Zm = (start' * ones(1, steps) + (Z-start)'*t)';

            s = surf(Xm, Ym, Zm, varargin{:});
        end        
        function v = checkValidity(F1inv, F3inv, p1, p2, q1, q2)
            A = [p1, q1];
            B = [p2, q1];
            C = [p2, q2];
            D = [p1, q2];
            % Check if each point of the rectangle is in the Feasibility Region
            inFR = @(pt) (F3inv(1-pt(2)) >= F1inv(pt(1)));
            inFeasReg = [inFR(A), inFR(B), inFR(C), inFR(D)];
            v = (sum(inFeasReg) == 4);
        end
        
        function [pvest, isvalid] = calcPVUS(F1, F2, F3, F1inv, F3inv, f2, p1, p2, q1, q2)
            if pVUS.checkValidity(F1inv, F3inv, p1, p2, q1, q2)
                pvest = (p2-p1).*(q2-q1).*(F2(F3inv(1-q2)) - F2(F1inv(p2)))...
                    + (p2-p1).*quadgk(@(t) (1-F3(t) - q1).*f2(t), F3inv(1-q2), F3inv(1-q1))...
                    + (q2-q1).*quadgk(@(t) (F1(t) - p1).*f2(t), F1inv(p1), F1inv(p2));
                isvalid = 1;
            else
                pvest = 0;
                isvalid = 0;
            end
        end
        
        function [pvusH0, isvalid] = calcPVUSH0(p1, p2, q1, q2)
            if q2 <= 1-p2 || (p2+q2-1) < 1e-10
                pvusH0 = (p2 - p1).*(q2 - q1).*(2-q2-q1-p2-p1)./2;
                isvalid = 1;
            else
                pvusH0 = 0;
                isvalid = 0;
            end
        end
    end
end

