classdef pVUSest < pVUS
    %UNTITLED2 Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
% Measurement values by group
        X1
        X2
        X3
% Sample Sizes by group
        n1
        n2
        n3
% Estimation Method
        method
        pvus_ci
        vvar
        valid_boot_reg
        lambda
        J
%  Transformed Measurements
        X1t
        X2t
        X3t
% Bandwidths for KDE Estimates
        bw1
        bw2
        bw3
        add_boxcox
        add_error
        prc_tile
        pvus_kde_boot
        Zstat
        pval
    end
    
    methods
        
        function obj = pVUSest(X1, X2, X3, p1, p2, q1, q2, method, outpt, gr_sz)
         % INPUTS:
         %   X1: Observed Measurements for "Healthy" Population
         %   X2: Observed Measurements for "Mildly Diseased" Population
         %   X3: Observed Measurements for "Diseased" Population
         %   p1: Lower bound for TC3
         %   p2: Upper bound for TC3
         %   q1: Lower bound for TC2
         %   q2: Upper bound for TC2
         %   method: pVUS estimation method
            if nargin < 9
                outpt = true;
            end
% Calculate Sammple sizes

            n1 = length(X1);
            n2 = length(X2);
            n3 = length(X3);
%-------------------------------------------------------------------------%
% Normal Method
            if strcmp(method, 'normal')
                [m1, s1, m2, s2, m3, s3,...
                 a, b, c, d,varcov_p,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitNormals(X1,X2,X3);
                add_boxcox = false;
% Estimate pVUS
                vest = (p2-p1).*(q2-q1).*(normcdf((d-norminv(q2))./c) - normcdf((b+norminv(p2))./a))...
                    + (p2-p1).*integral(@(t) (normcdf(-c.*t + d) - q1).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c)...
                    + (q2-q1).*integral(@(t) (normcdf(a.*t - b) - p1).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a);
% Estimate Derivatives wrt f1 and f2
                df2_dm1 =-1/s1;
                df1_dm2 = -1/s3; 
                df2_dm2 =1/s1;     
                df1_dm3 = 1/s3; 
                df2_ds1 = @(t) (m1 - m2)/s1^2 - (s2*t)/s1^2;    
                df1_ds2 = @(t) -t/s3; 
                df2_ds2 = @(t) t/s1; 
                df1_ds3 = @(t) (m2 - m3)/s3^2 + (s2*t)/s3^2; 
% Estimate Derivatives wrt pVUS
                dVpq_ds1 = (q2-q1).*quadgk(@(t) df2_ds1(t).*normpdf(a*t - b).*  normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a); 
                dVpq_dm1 = (q2-q1).*quadgk(@(t)    df2_dm1.*normpdf(a*t - b).*  normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a);
                dVpq_ds2 = (q2-q1).*quadgk(@(t) df2_ds2(t).*normpdf( a.*t - b).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a)...
                     + (p2-p1).*quadgk(@(t) df1_ds2(t).*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dVpq_dm2 = (q2-q1).*quadgk(@(t)    df2_dm2.*normpdf(a.*t - b).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a)...
                     + (p2-p1).*quadgk(@(t)    df1_dm2.*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dVpq_ds3 = (p2-p1).*quadgk(@(t) df1_ds3(t).*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dVpq_dm3 = (p2-p1).*quadgk(@(t)    df1_dm3.*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                J = [dVpq_dm1 dVpq_ds1 dVpq_dm2 dVpq_ds2 dVpq_dm3 dVpq_ds3];
% Calculate Variance of pVUS Estimate
                vvar = J*varcov_p*J';
% Calculate 95% confidence intervals
                z = norminv(0.975);
                pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
                X1t = X1; X2t = X2; X3t = X3;
            end 
%-------------------------------------------------------------------------%
% Box-Cox Method
            if strcmp(method, 'boxcox')
                [m1, s1, m2, s2, m3, s3,a, b, c, d,varcov_p,...
                X1t, X2t, X3t,lambda,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitBoxCox(X1,X2,X3); 
 % Estimate pVUS
                vest = (p2-p1).*(q2-q1).*(normcdf((d-norminv(q2))./c) - normcdf((b+norminv(p2))./a))...
                    + (p2-p1).*integral(@(t) (normcdf(-c.*t + d) - q1).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c)...
                    + (q2-q1).*integral(@(t) (normcdf(a.*t - b) - p1).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a);
% Estimate Derivatives wrt f1 and f2
                df2_dm1 = -1/s1;
                df1_dm2 = -1/s3; 
                df2_dm2 = 1/s1;     
                df1_dm3 = 1/s3; 
                df2_ds1 = @(t) (m1 - m2)/s1^2 - (s2*t)/s1^2;    
                df1_ds2 = @(t) -t/s3; 
                df2_ds2 = @(t) t/s1; 
                df1_ds3 = @(t) (m2 - m3)/s3^2 + (s2*t)/s3^2; 
% Estimate Derivatives wrt pvus
                dpv_ds1 = (q2-q1).*quadgk(@(t) df2_ds1(t).*normpdf(a*t - b).*  normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a); 
                dpv_dm1 = (q2-q1).*quadgk(@(t)    df2_dm1.*normpdf(a*t - b).*  normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a);
                dpv_ds2 = (q2-q1).*quadgk(@(t) df2_ds2(t).*normpdf( a.*t - b).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a)...
                     + (p2-p1).*quadgk(@(t) df1_ds2(t).*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dpv_dm2 = (q2-q1).*quadgk(@(t)    df2_dm2.*normpdf( a.*t - b).*normpdf(t), (b+norminv(p1))./a, (b+norminv(p2))./a)...
                     + (p2-p1).*quadgk(@(t)    df1_dm2.*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dpv_ds3 = (p2-p1).*quadgk(@(t) df1_ds3(t).*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                dpv_dm3 = (p2-p1).*quadgk(@(t)    df1_dm3.*normpdf(-c.*t + d).*normpdf(t), (d-norminv(q2))./c, (d-norminv(q1))./c); 
                J = [dpv_dm1 dpv_ds1 dpv_dm2 dpv_ds2 dpv_dm3 dpv_ds3];
% Estimate variance of pvus estimate
                vvar = J*varcov_p*J';
% Calculate 95% confidence intervals
                z = norminv(0.975);
                pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
            end
%-------------------------------------------------------------------------%
% KDE Methods
            if strcmp(method, 'kde_sym') ||...
                    strcmp(method, 'kde_err_sym') ||...
                    strcmp(method, 'kde_err_prctile') ||...
                    strcmp(method, 'kde_prctile') ||...
                    strcmp(method, 'bc_kde_sym') ||...
                    strcmp(method, 'bc_kde_err_sym') ||...
                    strcmp(method, 'bc_kde_err_prctile') ||...
                    strcmp(method, 'bc_kde_prctile')
                method_split = split(method, '_');
                
                % Parse method string
                add_boxcox = strcmp(method_split{1}, 'bc');
                prc_tile = strcmp(method_split{length(method_split)}, 'prctile');
                if add_boxcox
                    add_error = strcmp(method_split{3}, 'err');
                else 
                    add_error = strcmp(method_split{2}, 'err');
                end
                if add_boxcox && add_error
                    iqr_1 = iqr(X1);
                    iqr_2 = iqr(X2);
                    iqr_3 = iqr(X3);
                    mn = min([X1; X2; X3]);
                    c = abs(mn) + max([iqr_1; iqr_2; iqr_3]);
                else
                    c = 0;
                end
                [vest, isvalid, bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,...
                    F1inv,F2inv,F3inv, X1t, X2t, X3t, lambda]  = pVUSest.pVUSKDEest(X1, X2, X3, p1, p2, q1, q2, add_boxcox, false, 0);
% Estimate pVUS Variance and Confidence Interval
                if isvalid == 1
                    B = 400;                    % Set Number of Bootstrap iterations
                    pvus_kde_boot = zeros(B,1);  % Initialize Bootstrap sample vector
                    valid_reg_boot = zeros(B,1);
                    if outpt
                        fprintf('Generating Bootstrap Estimates\n');
                    end
% START BOOTSTRAP
                    for b = 1:B
                        if outpt
                            fprintf('.')
                            if mod(b, 50) == 0
                                fprintf('\n');
                            end
                        end
% Sample with replacement from each population 
                        X1b = randsample(X1, n1, true);
                        X2b = randsample(X2, n2, true);
                        X3b = randsample(X3, n3, true); 
% Calculate pvus Estimates using Bootstrap samples
                        [pvus_kde_boot(b,1), valid_reg_boot(b,1)] = pVUSest.pVUSKDEest(X1b, X2b,...
                            X3b, p1, p2, q1, q2, add_boxcox, add_error, c);
                    end
% END BOOTSTRAP
                    vvar = var(pvus_kde_boot);
                    valid_boot_reg = mean(valid_reg_boot); % pct. of valid bootstraps
                    if prc_tile
                        % Calculate variance and 95% confidence intervals
                        % using Percentiles
                        pvus_ci = [prctile(pvus_kde_boot,2.5), prctile(pvus_kde_boot,97.5)];   % Upper Conf. Limit
                    else
                    % Calculate variance and 95% confidence intervals using
                    % variance
                        z = norminv(0.975);
                        pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
                    end
                end
            end
            if nargin < 10
                gr_sz = 0.0025;
            end
            obj@pVUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,p1,p2,q1,q2,gr_sz);
            obj.method = method;
            obj.X1 = X1; obj.X2 = X2; obj.X3 = X3;
            obj.n1 = n1; obj.n2 = n2; obj.n3 = n3;
            if strcmp(method, 'normal') || strcmp(method, 'boxcox')
                obj.J = J;
            end
            if obj.isvalid
                obj.vvar = vvar;
                obj.pvus_ci = pvus_ci;
                if strcmp(method, 'boxcox') || add_boxcox
                    obj.X1t = X1t;
                    obj.X2t = X2t;
                    obj.X3t = X3t;
                    obj.lambda = lambda;
                end
                if strcmp(method(1:3), 'kde') || strcmp(method(4:6), 'kde')
                    obj.bw1 = bw1; obj.bw2 = bw2; obj.bw3 = bw3;
                    obj.valid_boot_reg = valid_boot_reg;
                    obj.add_boxcox = add_boxcox;
                    obj.add_error = add_error;
                    obj.prc_tile = prc_tile;
                    obj.pvus_kde_boot = pvus_kde_boot;
                end
                if obj.isvalidH0 && obj.isvalid
                    obj.Zstat = (obj.pvus - obj.pvusH0)./sqrt(obj.vvar);
                    obj.pval = (1 - normcdf(abs(obj.Zstat))).*2;
                end
            end
        end
    end
    
    methods(Static)
%-------------------------------------------------------------------------%
% Estimation of the KDE's for the 3 groups
        function [vest, isValid,bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,...
              F1inv,F2inv,F3inv, X1t, X2t, X3t, lambda] = pVUSKDEest(X1, X2, X3, p1, p2, q1, q2, add_boxcox, add_error, c)
            n1 = length(X1); n2 = length(X2); n3 = length(X3);
            if nargin < 7
                add_boxcox = false;
                add_error = false;
                c = 0;
            end 
            % Box-Cox transform 
            if add_boxcox
                [lambda, X1t, X2t, X3t] = boxcox_roc3(X1, X2, X3); 
            else
                X1t = X1; X2t = X2; X3t = X3;   
                lambda = 0;
            end
            if add_error
                bw1= 0.9*min([std(X1t) iqr(X1t)/1.34])*n1^(-0.2);
                bw2= 0.9*min([std(X2t) iqr(X2t)/1.34])*n2^(-0.2);
                bw3= 0.9*min([std(X3t) iqr(X3t)/1.34])*n3^(-0.2);
                X1t = X1t + normrnd(0, bw1, n1, 1);
                X2t = X2t + normrnd(0, bw2, n2, 1);
                X3t = X3t + normrnd(0, bw3, n3, 1);
                X1t = X1t + c;
                X2t = X2t + c;
                X3t = X3t + c;
            end

            [bw1, bw2, bw3,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitKDE(X1t,X2t,X3t); 
            [vest, isValid] = pVUS.calcPVUS(F1, F2, F3, F1inv,F3inv, f2, p1, p2, q1, q2);
        end
    end
end

