classdef pVUStc1 < VUS

    properties
        p1
        p2
        q1
        q2
        
        tc2gr_q
        tc3gr_p
        
        pvus
        isvalid
        
        pvusH0
        isvalidH0
    end
    
    methods
%-------------------------------------------------------------------------%
% Initialize instance of pVUStc1 Class
        function obj = pVUStc1(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,p1,p2,q1,q2,gr_sz)
         % INPUTS:
         %   X1: Observed Measurements for "Healthy" Population
         %   X2: Observed Measurements for "Mildly Diseased" Population
         %   X3: Observed Measurements for "Diseased" Population
         %   p1: Lower bound for TC3
         %   p2: Upper bound for TC3
         %   q1: Lower bound for TC2
         %   q2: Upper bound for TC2
         %   method: pVUS estimation method
            if nargin < 20; gr_sz = 0.005; end
            obj@VUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,gr_sz);

            
            obj.p1 = p1;
            obj.p2 = p2;
            obj.q1 = q1;
            obj.q2 = q2;
            
            [obj.pvusH0, obj.isvalidH0] = pVUStc1.calcPVUSH0(p1, p2, q1, q2);
            [obj.pvus, obj.isvalid] = pVUStc1.calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2);
            if gr_sz < 1
                tc1 = @(y, z) obj.F1(obj.F2inv(obj.F2(obj.F3inv(1-y)) -z));
                % Set partial grid for plotting partial ROC Surface
                obj.tc3gr_p = [obj.p1:gr_sz:obj.p2 obj.p2];
                obj.tc2gr_q = [obj.q1:gr_sz:obj.q2 obj.q2];
            end
        end
%-------------------------------------------------------------------------%        
% Plot boundaries of pVUS on the x-y plane        
        function plotBounds(obj, varargin)
            plot3([0 0], [obj.p1 obj.p1], [0 1], varargin{:})
            plot3([0 0], [obj.p2 obj.p2], [0 1], varargin{:})
            plot3([0 0], [0 1], [obj.q1 obj.q1], varargin{:})
            plot3([0 0], [0 1], [obj.q2 obj.q2], varargin{:})
        end
%-------------------------------------------------------------------------% 
% Plot outline of floor
        function plotBoundFloor(obj, varargin)
            surf([0 0], [0 1], [0 1; 0 1], varargin{:})    
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function  pl = plotSurf(obj, varargin)
            tc3 = linspace(obj.p1, obj.p2, 1./obj.gr_sz);
            tc2 = linspace(obj.q1, obj.q2, 1./obj.gr_sz);
            tc1 = @(y, z) obj.F1(obj.F2inv(obj.F2(obj.F3inv(1-y)) -z));
            [Ym, Zm] = meshgrid(tc3, tc2);
            sz = size(Ym);
            X = tc1(reshape(Ym, 1, []), reshape(Zm, 1, []));
            Xsurf = reshape(X, sz(1), sz(2));
            pl = surf(Xsurf, Ym, Zm, varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p, r] = plotSurfWithROC(obj, varargin)
            tc1 = @(y, z) obj.F1(obj.F2inv(obj.F2(obj.F3inv(1-y)) -z)).*(obj.F2inv(z) <= obj.F3inv(1-y));
            p = obj.plotSurf(varargin{:});
            y = unique([0:obj.gr_sz:obj.p1, obj.p1:obj.gr_sz:obj.p2, obj.p2:obj.gr_sz:1, 1]);
            z = unique([0:obj.gr_sz:obj.q1, obj.q1:obj.gr_sz:obj.q2, obj.q2:obj.gr_sz:1, 1]);
            [y_m, z_m] = meshgrid(y,z);
            x = tc1(y_m,z_m);
            x(isnan(x)) = -0.01;
            x(z > obj.q1 & z < obj.q2, y > obj.p1 & y < obj.p2) = NaN;
            r = surf(x, y_m, z_m,'EdgeColor', 'none', 'FaceLighting', 'none');
            r.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function  pl = plotSurfH0(obj, varargin)
            Y = [obj.p1; obj.p1; obj.p2; obj.p2];
            Z = [obj.q1; obj.q2; obj.q2; obj.q1];
            X = 1-Y-Z;
            pl = patch(X,Y,Z,'k', varargin{:})
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p, r] = plotSurfWithROCH0(obj, varargin)
            p = obj.plotSurfH0(varargin{:});
            Y = [1; 0; 0; obj.p1; obj.p1; obj.p2; obj.p2; obj.p1; 0];
            Z = [0; 1; 0; obj.q1; obj.q2; obj.q2; obj.q1; obj.q1; 0];
            X = 1-Z-Y;
            r = patch(X,Y,Z,Z, 'EdgeColor','none','FaceLighting','none', 'FaceAlpha', 0.8);
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%
% Plot the Outline of the pVUS
        function plotOutline(obj, varargin)
            tc1 = @(y, z) obj.F1(obj.F2inv(obj.F2(obj.F3inv(1-y)) -z));
            gr = 0.01;
            % Line on ROC Surface from (p1,q1) to (p2,q1)
                X1 = [obj.p1:gr:obj.p2, obj.p2];
                Y1 = X1.*0 + obj.q1;
            % Line on ROC Surface from (p1,q2) to (p2,q2) 
                X2 = [obj.p1:gr:obj.p2, obj.p2];
                Y2 = X2.*0 + obj.q2;
            % Line on ROC Surface from (p1,q1) to (p1,q2)
                Y3 = [obj.q1:gr:obj.q2, obj.q2];
                X3 = Y3.*0 + obj.p1;
            % Line on ROC Surface from (p2,q1) to (p2,q2)
                Y4 = [obj.q1:gr:obj.q2, obj.q2];
                X4 = Y4.*0 + obj.p2;
            hold on;
            plot3(tc1(X1, Y1), X1, Y1, varargin{:});
            plot3(tc1(X2, Y2), X2, Y2, varargin{:});
            plot3(tc1(X3, Y3), X3, Y3, varargin{:});
            plot3(tc1(X4, Y4), X4, Y4, varargin{:});
            plot3([0 0], [obj.p1, obj.p2], [obj.q1, obj.q1], varargin{:});
            plot3([0 0], [obj.p1, obj.p1], [obj.q1, obj.q2], varargin{:});
            plot3([0 0], [obj.p1, obj.p2], [obj.q2, obj.q2], varargin{:});
            plot3([0 0], [obj.p2, obj.p2], [obj.q1, obj.q2], varargin{:});
            plot3([0 tc1(obj.p1, obj.q1)], [obj.p1, obj.p1], [obj.q1, obj.q1], varargin{:});
            plot3([0 tc1(obj.p1, obj.q2)], [obj.p1, obj.p1], [obj.q2, obj.q2], varargin{:});
            plot3([0 tc1(obj.p2, obj.q1)], [obj.p2, obj.p2], [obj.q1, obj.q1], varargin{:});
            plot3([0 tc1(obj.p2, obj.q2)], [obj.p2, obj.p2], [obj.q2, obj.q2], varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function plotSides(obj, varargin)
            tc1 = @(y, z) obj.F1(obj.F2inv(obj.F2(obj.F3inv(1-y)) -z));
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            Y1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Z1 = [obj.q1; obj.q1; p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            X1 = [0; 0; tc1(p_gr',obj.q1); 0; 0];
            Z2 = [obj.q2; obj.q2; p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            X2 = [0; 0; tc1(p_gr',obj.q2); 0; 0];
            Z3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            Y3 = [obj.p1; obj.p1; q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            X3 = [0; 0; tc1(obj.p1,q_gr'); 0; 0];
            Y4 = [obj.p2; obj.p2; q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            X4 = [0; 0; tc1(obj.p2,q_gr'); 0; 0];
            patch([X1, X2, X3,X4], [Y1, Y1, Y3,Y4], [Z1,Z2,Z3,Z3], 'k', varargin{:})
            axis([0 1 0 1 0 1])
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function plotSidesToROCH0(obj, varargin)
            tc1 = @(y, z) 1 - y -z;
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            Y1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Z1 = [obj.q1; obj.q1; p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            X1 = [0; 0; tc1(p_gr',obj.q1); 0; 0];
            Z2 = [obj.q2; obj.q2; p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            X2 = [0; 0; tc1(p_gr',obj.q2); 0; 0];
            Z3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            Y3 = [obj.p1; obj.p1; q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            X3 = [0; 0; tc1(obj.p1,q_gr'); 0; 0];
            Y4 = [obj.p2; obj.p2; q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            X4 = [0; 0; tc1(obj.p2,q_gr'); 0; 0];
            patch([X1, X2, X3,X4], [Y1, Y1, Y3,Y4], [Z1,Z2,Z3,Z3], 'k', varargin{:})
            axis([0 1 0 1 0 1])
        end
%-------------------------------------------------------------------------%
% Plot the pVUS region on the x-y plane
        function plotFloor(obj, varargin)
            gr = 0.01;
            Y = [obj.p1:gr:obj.p2, obj.p2];
            Z = [obj.q1:gr:obj.q2, obj.q2]; 
            [Ym, Zm] = meshgrid(Y, Z);
            surf(zeros(size(Ym)), Ym, Zm, varargin{:});
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region      
        function pl = plotFeasReg(obj, varargin)
            Y = obj.tc3gr;
            X = Y.*0;
            Z = obj.F2(obj.c2gr);
            [Zm, Xm] = meshgrid(Z,X);
            steps = max(size(Y));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Ym = (start' * ones(1, steps) + Y'*t)';

            pl = surf(Xm, Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
%             surf(zeros(size(Y)), Ym, Zm, varargin{:});
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region for the uninformative ROC Surface     
        function pl = plotFeasRegH0(obj, varargin)
            Y = obj.tc3gr;
            X = Y.*0;
            Z = 1-obj.tc3gr;
            [Zm, Xm] = meshgrid(Z,X);
            steps = max(size(Y));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Ym = (start' * ones(1, steps) + Y'*t)';

            surf(Xm, Ym, Zm, varargin{:})
            pl = surf(zeros(size(Y)), Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasible region for Inference   
        function pl = plotFeasRegInf(obj, varargin)
            Y = obj.tc3gr;
            X = Y.*0;
            Z = obj.F2(obj.c2gr);
            [Zm, Xm] = meshgrid(Z,X);
            steps = max(size(Y));
            start = zeros(1,steps);
            

            t = linspace(0,1,steps);
            Ym = (start' * ones(1, steps) + Y'*t)';
            Zm(Zm > 1-Ym) = 1-Ym(Zm > 1-Ym);
            pl = surf(zeros(size(Y)), Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region on the x-y plane        
        function  plotFeasRegFlat(obj, varargin)
            x = obj.tc3gr;
            area(x, 1-obj.F3(x), varargin{:});
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_2$','Interpreter', 'Latex'); 
        end
        function plotFeasRegOutline(obj, varargin)
            x = obj.tc1gr;
            plot3(x.*0, x, obj.F2(obj.c2gr), varargin{:});
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end

%-------------------------------------------------------------------------%
% Plot one side given values of x, y, and z (helper function for plotSides) 
        function plotSide(obj, X, Y, Z, varargin)
            if length(unique(Y)) == 1
                [Zm, Ym] = meshgrid(Z, Y);
            else 
                [Ym, Zm] = meshgrid(Y,Z);
            end

            steps = max(size(X));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Xm = (start' * ones(1, steps) + X'*t)';

            surf(Xm, Ym, Zm, varargin{:})
        end
        function plotROCandpVUS(obj)
            obj.plotROC('EdgeColor', 'none', 'FaceAlpha', 1)
            hold on
            obj.plotROCOutline('LineWidth', 1, 'Color', [1 1 1].*0.3)
            obj.plotSides('EdgeColor','none','FaceAlpha',0.6,'FaceColor','#9819E6')
            obj.plotFloor('EdgeColor','none','FaceColor','#420B63')
            obj.plotSurf(true, 'EdgeColor', 'none','FaceColor','#2A0740')
            obj.plotOutline('LineWidth', 1, 'Color', '#2D0845')
            obj.plotBounds('--k', 'Color', 'black') 
            light('Position',[0 0 2]);
            light('Position',[0 0 0.5],'Style','local');
            light('Position',[0 1 0.5],'Style','local');
        end
    end
    methods(Static)
%-------------------------------------------------------------------------%
% Check the Validity of the pVUS region
        function v = checkValidity(F2inv, F3inv, p1, p2, q1, q2)
            S3inv = @(t) F3inv(1-t);
            
            % Set vertices of the rectangle
            A = [p1, q1];
            B = [p2, q1];
            C = [p2, q2];
            D = [p1, q2];
            
            % Check if each point of the rectangle is in the Feasibility Region
            inFR = @(pt) (F2inv(pt(2)) <= S3inv(pt(1)));
            inFeasReg = [inFR(A), inFR(B), inFR(C), inFR(D)];
            v = (sum(inFeasReg) == 4);
        end
        function [pvest, isvalid] = calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2)
            if pVUStc1.checkValidity(F1inv, F3inv, p1, p2, q1, q2)
                S3inv = @(t) F3inv(1-t);
                S3 = @(t) 1-F3(t);

                % Set bounds of integration
                Bnd1 = F2inv(F2(S3inv(p2)) - q2);
                Bnd2 = F2inv(F2(S3inv(p1)) - q2);
                Bnd3 = F2inv(F2(S3inv(p2)) - q1);
                Bnd4 = F2inv(F2(S3inv(p1)) - q1);
                
                L1 = Bnd1;
                L2 = min(Bnd2, Bnd3);
                L3 = max(Bnd2, Bnd3);
                L4 = Bnd4;
                
                % Set each part of the integral
                A = @(t) (p2 - S3(F2inv(F2(t)+q2))).*F1(t).*f2(t);
                if Bnd3 < Bnd2
                    B = @(t) (S3(F2inv(F2(t) + q1)) - S3(F2inv(F2(t) + q2))).*F1(t).*f2(t);
                else
                    B = @(t) (p2 - p1).*F1(t).*f2(t);
                end
                C = @(t) (S3(F2inv(F2(t)+q1)) - p1).*F1(t).*f2(t);
                if L1 == L2; A = @(t) 0; end
                if L3 == L4; C = @(t) 0; end
                
                % Integrate and add up all parts to calculate pVUS
                pvest = integral(A, L1, L2) + integral(B, L2, L3) + integral(C, L3, L4);
                isvalid = 1; % Flag as valid
            else
                pvest = 0;
                isvalid = 0; % Flag as invalid
            end
        end
        function [pvusH0, isvalid] = calcPVUSH0(p1, p2, q1, q2)
            if p2 <= 1-q2 || (p2+q2-1) < 1e-10
                pvusH0 = (p2 - p1).*(q2 - q1).*(2-q2-q1-p2-p1)./2;
                isvalid = 1;
            else
                pvusH0 = 0;
                isvalid = 0;
            end
        end
    end
end

